
/*

     Primitives for reading a qcd data file generated by my program.

*/

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/file.h>
#include "spnhdr.h"
#include "strmhdr.h"

/* microseconds */
/*
#define DELAY 200000
*/

extern float get_float () ;
extern int   get_int   () ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

static void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "dsource: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

DATA *data_dopen_read (fildes)
int fildes ;
{
  DATA *result ;

  result = (DATA *) malloc (sizeof (DATA)) ;

  result->desc = fildes ;
  result->save_full = 1 ;

  return result ;
}

DATA *data_open_read (file)
char *file ;
{
  int fildes ;

  if (file == NULL)
    fildes = 0 ;
  else
  {
    char err_mess [128] ;

    fildes = open (file, O_RDONLY, 0) ;

    strcpy (err_mess, "Failed to open \"") ;
    strcat (err_mess, file) ;
    strcat (err_mess, "\"") ;
    sys_assert (fildes >= 0, err_mess) ;
  }

  return data_dopen_read (fildes) ;
}

void data_close_read (data)
DATA *data ;
{
  int result ;

  result   = read (data->desc, data->buf, BLOCK_SIZE) ;
  if (result != 0)
    fprintf (stderr, "warning: file closed before eof encountered\n") ;

  close (data->desc) ;
  free  (data) ;
}

static void read_buffer (data, len_to_read)
DATA *data ;
int len_to_read ;
{
  int ptr, residue ;

  ptr     = 0 ;
  residue = len_to_read ;
  while (residue > 0)
  {
    int result ;

    result   = read (data->desc, (data->buf) + ptr, residue) ;
    sys_assert (result > 0, "Read failed") ;
    ptr     += result ;
    residue -= result ;
  }
}

void read_header (data)
DATA *data ;
{
  read_buffer (data, BLOCK_SIZE) ;
  /*
  int result ;

  result = read (data->desc, buf, BLOCK_SIZE) ;
  sys_assert (result == BLOCK_SIZE, "Read failed") ;
  */
}

void read_coeffs (data, coeffs)
DATA *data ;
float coeffs [] ;
{
  /*
  int buf [BLOCK_SIZE / sizeof (int)] ;
  int result ;

  result = read (data->desc, (char *) buf, BLOCK_SIZE) ;
  sys_assert (result == BLOCK_SIZE, "Read failed") ;
  */
  int *buf ;

  read_buffer (data, BLOCK_SIZE) ;
  /* fprintf (stderr, "read buffer for coeffs\n") ; */
  buf = (int *) data->buf ;

  {
    int num_coeffs ;
    int i ;

    num_coeffs = get_int (buf [0]) ;
    for (i = 0; i < num_coeffs; i++)
      coeffs [i] = get_float (buf [i + 1]) ;
  }
}

void read_params (data, params)
DATA *data ;
int params [] ;
{
  /*
  int buf [BLOCK_SIZE / sizeof (int)] ;
  int result ;

  result = read (data->desc, (char *) buf, BLOCK_SIZE) ;
  sys_assert (result == BLOCK_SIZE, "Read failed") ;
  */
  int *buf ;

  read_buffer (data, BLOCK_SIZE) ;
  /* fprintf (stderr, "read buffer for params\n") ; */
  buf = (int *) data->buf ;

  {
    int num_params ;
    int i ;

    num_params = get_int (buf [0]) ;
    for (i = 0; i < num_params; i++)
      params [i] = get_int (buf [i + 1]) ;
  }
}

void read_field_begin (data)
DATA *data ;
{
  data->site_size        = sizeof (float) * N_SPIN * N_COL * 2 ;
  data->sites_per_buffer = BUFFER_SIZE / data->site_size ;
  data->buf_ptr          = data->sites_per_buffer ;
}

void read_field_jump  (data, site_no)
DATA *data ;
int site_no ;
{
  /* This routine can only be used if the source data file is a disk file */

  int data_block_no = site_no / data->sites_per_buffer ;
  int site_offset   = site_no % data->sites_per_buffer ;
  int header_size   = 3 * BLOCK_SIZE ;
  int buffer_size   = data->save_full ?
		        BUFFER_SIZE :
		        data->site_size * data->sites_per_buffer ;
  off_t file_offset = header_size + (buffer_size * data_block_no) ;

  lseek (data->desc, file_offset, L_SET) ;
  read_buffer (data, buffer_size) ;
  data->buf_ptr = site_offset ;
}

void read_field_record (data, field_record)
DATA *data ;
char **field_record ;
{
  if (data->buf_ptr >= data->sites_per_buffer)
  {
    if (data->save_full)
      read_buffer (data, BUFFER_SIZE) ;
    else
      read_buffer (data, data->site_size * data->sites_per_buffer) ;

    data->buf_ptr = 0 ;
  }
  *field_record = data->buf + (data->site_size * data->buf_ptr) ;
  (data->buf_ptr)++ ;
}

